==> e_commerce-0.0.12-sources.jar: ai/causalfoundry/android/sdk/e_commerce/builders/CfLogScheduleDeliveryEvent.kt <==
package ai.causalfoundry.android.sdk.e_commerce.builders

import ai.causalfoundry.android.sdk.core.CFSetup
import ai.causalfoundry.android.sdk.core.utils.CoreConstants
import ai.causalfoundry.android.sdk.core.utils.ExceptionManager
import ai.causalfoundry.android.sdk.e_commerce.event_models.ScheduleDeliveryObject
import ai.causalfoundry.android.sdk.e_commerce.event_types.EComEventType
import ai.causalfoundry.android.sdk.e_commerce.event_types.ScheduleDeliveryAction
import ai.causalfoundry.android.sdk.e_commerce.utils.ECommerceConstants

/**
 * Created by Moiz Hassan on 22 March, 2023
 */

class CfLogScheduleDeliveryEvent {

    /**
     * CfLogScheduleDeliveryEvent is used to log the status for the delivery. It can be used to log the
     * delivered status of the order or a partial order. Details about the items in the specific
     * delivery should be provided in the catalog.
     */
    data class Builder(
        internal var order_id: String? = null,
        internal var is_urgent: Boolean? = null,
        internal var action: String? = null,
        internal var delivery_ts: Long? = null,
        private var meta: Any? = null,
        private var update_immediately: Boolean = CoreConstants.updateImmediately
    ) {

        /**
         * setOrderId is required to associate the delivery event with the order. OrderId
         * should be a valid orderId and can be tracked from the checkout.
         */
        fun setOrderId(order_id: String) = apply { this.order_id = order_id }

        /**
         * isUrgent is to mark the log if the delivery is scheduled to be an immediate or emergency
         * delivery.
         */
        fun isUrgent(is_urgent: Boolean) = apply { this.is_urgent = is_urgent }

        /**
         * setScheduleDeliveryAction is required to set the delivery action for the log. For the order
         * being prepared for delivery that if the item is being scheduled or updated in terms of
         * delivery elements
         */
        fun setScheduleDeliveryAction(action: ScheduleDeliveryAction) =
            apply { this.action = action.name }

        fun setScheduleDeliveryAction(action: String) = apply {
            if (CoreConstants.enumContains<ScheduleDeliveryAction>(action)) {
                this.action = action
            } else {
                ExceptionManager.throwEnumException(ScheduleDeliveryAction::class.java.simpleName)
            }
        }

        /**
         * The timestamp in time in milliseconds format for the date and time for which the
         * delivery is set to be scheduled
         */
        fun setDeliveryDateTime(delivery_ts: Long) = apply { this.delivery_ts = delivery_ts }
        fun setDeliveryDateTime(delivery_ts_string: String) = apply {
            try {
                this.delivery_ts = delivery_ts_string.toLong()
            } catch (ex: java.lang.Exception) {
                ExceptionManager.throwRuntimeException("Unable to convert $delivery_ts_string to Long")
            }
        }

        /**
         * You can pass any type of value in setMeta. It is for developer and partners to log
         * additional information with the log that they find would be helpful for logging and
         * providing more context to the log. Default value for the meta is null.
         */
        fun setMeta(meta: Any?) = apply { this.meta = meta }

        /**
         * updateImmediately is responsible for updating the values ot the backend immediately.
         * By default this is set to false or whatever the developer has set in the SDK
         * initialisation block. This differs the time for which the logs will be logged, if true,
         * the SDK will log the content instantly and if false it will wait till the end of user
         * session which is whenever the app goes into background.
         */
        fun updateImmediately(update_immediately: Boolean) =
            apply { this.update_immediately = update_immediately }

        /**
         * build will validate all of the values provided and if passes will call the track
         * function and queue the events based on it's updateImmediately value and also on the
         * user's network resources.
         */
        fun build() = apply {
            when {

                /**
                 * Will throw and exception if the orderId provided is null or no value is
                 * provided at all.
                 */
                order_id.isNullOrEmpty() -> {
                    ExceptionManager.throwIsRequiredException("order_id")
                }

                /**
                 * Will throw and exception if the is_urgent provided is null or no value is
                 * provided at all.
                 */
                is_urgent == null -> {
                    ExceptionManager.throwIsRequiredException("is_urgent")
                }

                /**
                 * Will throw and exception if the delivery_action provided is null or no value is
                 * provided at all.
                 */
                action.isNullOrEmpty() -> {
                    ExceptionManager.throwIsRequiredException(ScheduleDeliveryAction::class.java.simpleName)
                }

                /**
                 * Will throw and exception if the delivery_ts provided is null or no value is
                 * provided at all.
                 */
                delivery_ts == null || delivery_ts == 0L -> {
                    ExceptionManager.throwIsRequiredException("delivery_ts")
                }

                else -> {

                    if (delivery_ts!! < System.currentTimeMillis() - 10000L) {
                        ExceptionManager.throwIllegalStateException("Scheduled time should be in future")
                    }

                    /**
                     * Parsing the values into an object and passing to the setup block to queue
                     * the event based on its priority.
                     */

                    val scheduleDeliveryObject = ScheduleDeliveryObject(
                        order_id!!, is_urgent!!, action!!,
                        ECommerceConstants.getDateTime(delivery_ts!!), meta
                    )
                    CFSetup().track(
                        ECommerceConstants.contentBlockName,
                        EComEventType.schedule_delivery.name,
                        scheduleDeliveryObject, update_immediately
                    )
                }
            }
        }
    }
}

==> e_commerce-0.2.4-sources.jar: ai/causalfoundry/android/sdk/e_commerce/builders/CfLogScheduleDeliveryEvent.kt <==
package ai.causalfoundry.android.sdk.e_commerce.builders

import ai.causalfoundry.android.sdk.core.CFSetup
import ai.causalfoundry.android.sdk.core.utils.CoreConstants
import ai.causalfoundry.android.sdk.core.utils.ExceptionManager
import ai.causalfoundry.android.sdk.e_commerce.event_models.event_objects.ScheduleDeliveryObject
import ai.causalfoundry.android.sdk.e_commerce.event_types.EComEventType
import ai.causalfoundry.android.sdk.e_commerce.event_types.ScheduleDeliveryAction
import ai.causalfoundry.android.sdk.e_commerce.utils.ECommerceConstants

/**
 * Created by Moiz Hassan on 22 March, 2023
 */

class CfLogScheduleDeliveryEvent {

    /**
     * CfLogScheduleDeliveryEvent is used to log the status for the delivery. It can be used to log the
     * delivered status of the order or a partial order. Details about the items in the specific
     * delivery should be provided in the catalog.
     */
    data class Builder(
        internal var order_id: String? = null,
        internal var is_urgent: Boolean? = null,
        internal var action: String? = null,
        internal var delivery_ts: Long? = null,
        private var meta: Any? = null,
        private var update_immediately: Boolean = CoreConstants.updateImmediately
    ) {

        /**
         * setOrderId is required to associate the delivery event with the order. OrderId
         * should be a valid orderId and can be tracked from the checkout.
         */
        fun setOrderId(order_id: String) = apply { this.order_id = order_id }

        /**
         * isUrgent is to mark the log if the delivery is scheduled to be an immediate or emergency
         * delivery.
         */
        fun isUrgent(is_urgent: Boolean) = apply { this.is_urgent = is_urgent }

        /**
         * setScheduleDeliveryAction is required to set the delivery action for the log. For the order
         * being prepared for delivery that if the item is being scheduled or updated in terms of
         * delivery elements
         */
        fun setScheduleDeliveryAction(action: ScheduleDeliveryAction) =
            apply { this.action = action.name }

        fun setScheduleDeliveryAction(action: String) = apply {
            if (CoreConstants.enumContains<ScheduleDeliveryAction>(action)) {
                this.action = action
            } else {
                ExceptionManager.throwEnumException(
                    EComEventType.schedule_delivery.name,
                    ScheduleDeliveryAction::class.java.simpleName
                )
            }
        }

        /**
         * The timestamp in time in milliseconds format for the date and time for which the
         * delivery is set to be scheduled
         */
        fun setDeliveryDateTime(delivery_ts: Long) = apply { this.delivery_ts = delivery_ts }
        fun setDeliveryDateTime(delivery_ts_string: String) = apply {
            try {
                this.delivery_ts = delivery_ts_string.toLong()
            } catch (ex: java.lang.Exception) {
                ExceptionManager.throwRuntimeException(
                    EComEventType.schedule_delivery.name,
                    "Unable to convert $delivery_ts_string to Long"
                )
            }
        }

        /**
         * You can pass any type of value in setMeta. It is for developer and partners to log
         * additional information with the log that they find would be helpful for logging and
         * providing more context to the log. Default value for the meta is null.
         */
        fun setMeta(meta: Any?) = apply { this.meta = meta }

        /**
         * updateImmediately is responsible for updating the values ot the backend immediately.
         * By default this is set to false or whatever the developer has set in the SDK
         * initialisation block. This differs the time for which the logs will be logged, if true,
         * the SDK will log the content instantly and if false it will wait till the end of user
         * session which is whenever the app goes into background.
         */
        fun updateImmediately(update_immediately: Boolean) =
            apply { this.update_immediately = update_immediately }

        /**
         * build will validate all of the values provided and if passes will call the track
         * function and queue the events based on it's updateImmediately value and also on the
         * user's network resources.
         */
        fun build() = apply {
            when {

                /**
                 * Will throw and exception if the orderId provided is null or no value is
                 * provided at all.
                 */
                order_id.isNullOrEmpty() -> {
                    ExceptionManager.throwIsRequiredException(
                        EComEventType.schedule_delivery.name,
                        "order_id"
                    )
                }

                /**
                 * Will throw and exception if the is_urgent provided is null or no value is
                 * provided at all.
                 */
                is_urgent == null -> {
                    ExceptionManager.throwIsRequiredException(
                        EComEventType.schedule_delivery.name,
                        "is_urgent"
                    )
                }

                /**
                 * Will throw and exception if the delivery_action provided is null or no value is
                 * provided at all.
                 */
                action.isNullOrEmpty() -> {
                    ExceptionManager.throwIsRequiredException(
                        EComEventType.schedule_delivery.name,
                        ScheduleDeliveryAction::class.java.simpleName
                    )
                }

                /**
                 * Will throw and exception if the delivery_ts provided is null or no value is
                 * provided at all.
                 */
                delivery_ts == null || delivery_ts == 0L -> {
                    ExceptionManager.throwIsRequiredException(
                        EComEventType.schedule_delivery.name,
                        "delivery_ts"
                    )
                }

                else -> {

                    if (delivery_ts!! < System.currentTimeMillis() - 10000L) {
                        ExceptionManager.throwIllegalStateException(
                            EComEventType.schedule_delivery.name,
                            "Scheduled time should be in future"
                        )
                    }

                    /**
                     * Parsing the values into an object and passing to the setup block to queue
                     * the event based on its priority.
                     */

                    val scheduleDeliveryObject = ScheduleDeliveryObject(
                        order_id!!, is_urgent!!, action!!,
                        ECommerceConstants.getDateTime(delivery_ts!!), meta
                    )
                    CFSetup().track(
                        ECommerceConstants.contentBlockName,
                        EComEventType.schedule_delivery.name,
                        scheduleDeliveryObject, update_immediately
                    )
                }
            }
        }
    }
}
